# -*- coding: utf-8 -*-
"""
test_simple_compression
~~~~~~~~~~~~~~~~~~~~~~~~~

Tests for compression of single chunks.
"""
import brotlicffi

import pytest

from hypothesis import given, settings
from hypothesis.strategies import binary, integers, sampled_from, one_of


def test_roundtrip_compression_with_files(simple_compressed_file):
    """
    Roundtripping data through the compressor works correctly.
    """
    with open(simple_compressed_file[0], 'rb') as f:
        uncompressed_data = f.read()

    assert brotlicffi.decompress(
        brotlicffi.compress(uncompressed_data)
    ) == uncompressed_data


@pytest.mark.slow
@settings(deadline=None)
@given(
    chunk_size=integers(min_value=1, max_value=2**12),
    mode=sampled_from(list(brotlicffi.BrotliEncoderMode)),
    quality=integers(min_value=0, max_value=11),
    lgwin=integers(min_value=10, max_value=24),
    lgblock=one_of(
        integers(min_value=0, max_value=0),
        integers(min_value=16, max_value=24)
    ),
)
def test_streaming_compression(one_compressed_file,
                               chunk_size,
                               mode,
                               quality,
                               lgwin,
                               lgblock):
    """
    Confirm that the streaming compressor works as expected.
    """
    compressed_chunks = []
    c = brotlicffi.Compressor(
        mode=mode, quality=quality, lgwin=lgwin, lgblock=lgblock
    )
    with open(one_compressed_file, 'rb') as f:
        while True:
            next_data = f.read(chunk_size)
            if not next_data:
                break

            compressed_chunks.append(c.compress(next_data))

    compressed_chunks.append(c.finish())
    decompressed = brotlicffi.decompress(b''.join(compressed_chunks))
    with open(one_compressed_file, 'rb') as f:
        assert decompressed == f.read()


@pytest.mark.slow
@settings(deadline=None)
@given(
    chunk_size=integers(min_value=1, max_value=2**12),
    mode=sampled_from(list(brotlicffi.BrotliEncoderMode)),
    quality=integers(min_value=0, max_value=11),
    lgwin=integers(min_value=10, max_value=24),
    lgblock=one_of(
        integers(min_value=0, max_value=0),
        integers(min_value=16, max_value=24)
    ),
)
def test_streaming_compression_flush(one_compressed_file,
                                     chunk_size,
                                     mode,
                                     quality,
                                     lgwin,
                                     lgblock):
    """
    Confirm that the streaming compressor works as expected, including flushes
    after each chunk.
    """
    compressed_chunks = []
    c = brotlicffi.Compressor(
        mode=mode, quality=quality, lgwin=lgwin, lgblock=lgblock
    )
    with open(one_compressed_file, 'rb') as f:
        while True:
            next_data = f.read(chunk_size)
            if not next_data:
                break

            compressed_chunks.append(c.compress(next_data))
            compressed_chunks.append(c.flush())

    compressed_chunks.append(c.finish())
    decompressed = brotlicffi.decompress(b''.join(compressed_chunks))
    with open(one_compressed_file, 'rb') as f:
        assert decompressed == f.read()


@given(binary())
def test_compressed_data_roundtrips(s):
    assert brotlicffi.decompress(brotlicffi.compress(s)) == s


@given(binary())
def test_process_alias(s):
    c1 = brotlicffi.Compressor()
    c2 = brotlicffi.Compressor()
    d1 = brotlicffi.Decompressor()
    d2 = brotlicffi.Decompressor()
    s1 = c1.compress(s) + c1.finish()
    s2 = c2.process(s) + c2.finish()
    assert (
        (d1.decompress(s1) + d1.finish())
        == (d2.process(s2) + d2.finish())
        == s
    )


@pytest.mark.parametrize(
    "params",
    [
        {"mode": 52},
        {"quality": 52},
        {"lgwin": 52},
        {"lgblock": 52},
    ]
)
def test_bad_compressor_parameters(params):
    with pytest.raises(brotlicffi.error):
        brotlicffi.Compressor(**params)



# -*- coding: utf-8 -*-
"""
test_simple_decompression
~~~~~~~~~~~~~~~~~~~~~~~~~

Tests for decompression of single chunks.
"""
import brotlicffi

import pytest


def test_decompression(simple_compressed_file):
    """
    Decompressing files returns their original form using decompress.
    """
    with open(simple_compressed_file[0], 'rb') as f:
        uncompressed_data = f.read()

    with open(simple_compressed_file[1], 'rb') as f:
        compressed_data = f.read()

    assert brotlicffi.decompress(compressed_data) == uncompressed_data


def test_decompressobj(simple_compressed_file):
    with open(simple_compressed_file[0], 'rb') as f:
        uncompressed_data = f.read()

    with open(simple_compressed_file[1], 'rb') as f:
        compressed_data = f.read()

    o = brotlicffi.Decompressor()
    data = o.decompress(compressed_data)
    data += o.flush()
    data += o.finish()

    assert data == uncompressed_data


# `more_data_limit` allows testing `decompress(b'')` with and without a limit.
@pytest.mark.parametrize('more_data_limit', [100, None])
def test_decompressobj_with_output_buffer_limit(
    simple_compressed_file, more_data_limit
):
    """
    Test decompression with `output_buffer_limit` set.
    """
    with open(simple_compressed_file[0], 'rb') as f:
        uncompressed_data = f.read()

    with open(simple_compressed_file[1], 'rb') as f:
        compressed_data = f.read()

    o = brotlicffi.Decompressor()
    assert o.can_accept_more_data()
    small_limit = 100
    result = o.decompress(compressed_data, output_buffer_limit=small_limit)
    assert len(result) <= small_limit

    # Ensure `output_buffer_limit` of zero works.
    assert o.decompress(b'', output_buffer_limit=0) == b''

    if o._unconsumed_data:
        with pytest.raises(
            brotlicffi.error,
            match=(
                r"brotli: decoder process called with data when "
                r"'can_accept_more_data\(\)' is False"
            ),
        ):
            o.decompress(b'additional data')

    if not o.is_finished():
        assert not o.can_accept_more_data()

        # Continue decompressing with empty input.
        all_output = [result]
        while not o.can_accept_more_data() and not o.is_finished():
            more_output = o.decompress(
                b'', output_buffer_limit=more_data_limit
            )
            if more_data_limit is not None:
                assert len(more_output) <= more_data_limit
            all_output.append(more_output)
        assert o.can_accept_more_data() or o.is_finished()

        final_result = b''.join(all_output)
        assert final_result == uncompressed_data


def test_drip_feed(simple_compressed_file):
    """
    Sending in the data one byte at a time still works.
    """
    with open(simple_compressed_file[0], 'rb') as f:
        uncompressed_data = f.read()

    with open(simple_compressed_file[1], 'rb') as f:
        compressed_data = f.read()

    outdata = []
    o = brotlicffi.Decompressor()
    for i in range(0, len(compressed_data)):
        assert not o.is_finished()
        outdata.append(o.decompress(compressed_data[i:i+1]))

    assert o.is_finished()
    outdata.append(o.flush())
    assert o.is_finished()
    outdata.append(o.finish())
    assert o.is_finished()

    assert b''.join(outdata) == uncompressed_data


@pytest.mark.parametrize('exception_cls', [brotlicffi.Error, brotlicffi.error])
def test_streaming_decompression_fails_properly_on_garbage(exception_cls):
    """
    Garbage data properly fails decompression.
    """
    o = brotlicffi.Decompressor()
    with pytest.raises(exception_cls):
        o.decompress(b'some random garbage')
    assert not o.is_finished()


@pytest.mark.parametrize('exception_cls', [brotlicffi.Error, brotlicffi.error])
@pytest.mark.parametrize('bogus', (b'some random garbage', b'bogus'))
def test_decompression_fails_properly_on_garbage(bogus, exception_cls):
    """
    Garbage data properly fails decompression.
    """
    with pytest.raises(exception_cls):
        brotlicffi.decompress(bogus)



""" Test multithreaded sharing of Compressor and Decompressor instances

Originally written by Eugene Kliuchnikov at Google for the brotli
Python bindings under an MIT license.

"""
import queue
import random
import threading
import time

import brotlicffi


def make_compress_input(size):
    abc = [bytes([b]) for b in b"abcdefghijklmnopqrstuvwxyz"]
    abc_cap = [bytes([b]) for b in b"ABCDEFGHIJKLMNOPQRSTUVWXYZ"]
    num_words_by_len = [0, 25, 100, 175, 1700, 1000, 1000, 1000]
    word_set = set()
    rng = random.Random()
    rng.seed(0x4d3d3d3)
    words_by_len = [[]]
    for word_len in range(1, len(num_words_by_len)):
        num_words = num_words_by_len[word_len]
        words = []
        for _ in range(num_words):
            while True:
                word = b"".join(
                    [rng.choice(abc_cap)]
                    + [rng.choice(abc) for _ in range(word_len - 1)]
                )
                if word not in word_set:
                    word_set.add(word)
                    words.append(word)
                    break
        words_by_len.append(words)
    total_size = 0
    out = []
    while total_size < size:
        word_len = rng.choice(range(1, len(num_words_by_len)))
        word = rng.choice(words_by_len[word_len])
        total_size += len(word)
        out.append(word)
    return b"".join(out)


def _thread_compress(original, compressor, barrier, results):
    barrier.wait()
    compressed = compressor.process(original)
    compressed += compressor.finish()
    results.put(1)


def _thread_concurrent_process_compress(compressor, barrier, results):
    barrier.wait()
    time.sleep(.01)
    try:
        _ = compressor.process(b"whatever")
    except brotlicffi.error:
        results.put(2)


def _thread_concurrent_flush_compress(compressor, barrier, results):
    barrier.wait()
    time.sleep(.01)
    try:
        _ = compressor.flush()
    except brotlicffi.error:
        results.put(3)


def _thread_concurrent_finish_compress(compressor, barrier, results):
    barrier.wait()
    time.sleep(.01)
    try:
        _ = compressor.finish()
    except brotlicffi.error:
        results.put(4)


def test_compress_concurrency():
    original = make_compress_input(2 * 1024 * 1024)
    compressor = brotlicffi.Compressor(quality=9)
    results = queue.Queue()
    barrier = threading.Barrier(4)
    threads = []
    threads.append(
        threading.Thread(
            target=_thread_compress,
            args=(original, compressor, barrier, results)
        )
    )
    threads.append(
        threading.Thread(
            target=_thread_concurrent_process_compress,
            args=(compressor, barrier, results)
        )
    )
    threads.append(
        threading.Thread(
            target=_thread_concurrent_flush_compress,
            args=(compressor, barrier, results)
        )
    )
    threads.append(
        threading.Thread(
            target=_thread_concurrent_finish_compress,
            args=(compressor, barrier, results)
        )
    )
    for thread in threads:
        thread.start()
    for thread in threads:
        thread.join()
    assert sorted(list(results.queue)) == [1, 2, 3, 4]


def make_decompress_input(size):
    compressor = brotlicffi.Compressor(quality=1)
    prologue = compressor.process(b'b')
    prologue += compressor.flush()
    filler = compressor.process(b'c')
    filler += compressor.flush()
    epilogue = compressor.finish()
    return b''.join(
        [prologue] + [filler] * (size // len(filler)) + [epilogue])


def _thread_decompress(compressed, decompressor, barrier, results):
    barrier.wait()
    _ = decompressor.process(compressed)
    if decompressor.is_finished():
        results.put(1)


def _thread_concurrent_process(decompressor, barrier, results):
    barrier.wait()
    time.sleep(.001)
    try:
        _ = decompressor.process(b'')
    except brotlicffi.error:
        results.put(2)


def _thread_concurrent_can_accept_more_data(decompressor, barrier, results):
    barrier.wait()
    time.sleep(.001)
    try:
        _ = decompressor.can_accept_more_data()
    except brotlicffi.error:
        results.put(3)


def _thread_concurrent_is_finished(decompressor, barrier, results):
    barrier.wait()
    time.sleep(.001)
    try:
        _ = decompressor.is_finished()
    except brotlicffi.error:
        results.put(4)


def test_decompressor_concurrency():
    compressed = make_decompress_input(16 * 1024 * 1024)
    decompressor = brotlicffi.Decompressor()
    results = queue.Queue()
    barrier = threading.Barrier(4)
    threads = []
    threads.append(
        threading.Thread(
            target=_thread_decompress,
            args=(compressed, decompressor, barrier, results)
        )
    )
    threads.append(
        threading.Thread(
            target=_thread_concurrent_process,
            args=(decompressor, barrier, results)
        )
    )
    threads.append(
        threading.Thread(
            target=_thread_concurrent_can_accept_more_data,
            args=(decompressor, barrier, results),
        )
    )
    threads.append(
        threading.Thread(
            target=_thread_concurrent_is_finished,
            args=(decompressor, barrier, results)
        )
    )
    for thread in threads:
        thread.start()
    for thread in threads:
        thread.join()
    assert sorted(list(results.queue)) == [1, 2, 3, 4]



# -*- coding: utf-8 -*-
import os
import os.path

import pytest

# Statically look for all the files.
curdir = os.path.split(os.path.dirname(__file__))[0]
TEST_DATA_DIR = os.path.join(curdir, 'libbrotli', 'tests', 'testdata')
simple_test_files = [
    os.path.join(TEST_DATA_DIR, p) for p in os.listdir(TEST_DATA_DIR)
]
compressed_simple_test_files = set(
    f for f in simple_test_files if f.endswith('.compressed')
)


@pytest.fixture(params=compressed_simple_test_files)
def simple_compressed_file(request):
    """
    Returns a two-tuple: (uncompressed_file, compressed_file), from the Brotli
    test fixtures.
    """
    file = request.param
    return (file[:-len('.compressed')], file)


@pytest.fixture(scope='session')
def one_compressed_file():
    return os.path.join(TEST_DATA_DIR, 'alice29.txt')



#!/usr/bin/env python
import os
import re
import platform
import sys
import sysconfig
from setuptools import find_packages, setup
from setuptools.command.build_ext import build_ext

base_dir = os.path.dirname(os.path.abspath(__file__))
long_description = (
    open("README.rst").read() + '\n\n' + open("HISTORY.rst").read()
)

with open(os.path.join(base_dir, "src", "brotlicffi", "__init__.py")) as f:
    __version__ = re.search(r"__version__ = \"([^\"]+)\"", f.read()).group(1)


class BuildClibBeforeExt(build_ext):
    """ Setuptools `develop` command (used by `pip install -e .`) only calls
    `build_ext`, unlike `install` which calls `build` and all its related
    sub-commands. Linking the CFFI extension with the libbrotli static library
    fails since the `build_clib` command is not called in the former case.

    This custom `build_ext` class ensures that `build_clib` command is run
    before the CFFI extension module is compiled.

    https://github.com/pypa/pip/issues/4523
    """

    def get_source_files(self):
        filenames = build_ext.get_source_files(self)
        filenames.extend(depends)
        return filenames

    def run(self):
        self.run_command("build_clib")
        build_ext.run(self)


depends = []
libraries = []
USE_SHARED_BROTLI = os.environ.get("USE_SHARED_BROTLI")
if USE_SHARED_BROTLI != "1":
    sources = []

    for root, _, filenames in os.walk("libbrotli/c"):
        root_parts = os.path.split(root)
        if "fuzz" in root_parts or "tools" in root_parts:
            continue
        for filename in filenames:
            relpath = os.path.relpath(os.path.join(root, filename), base_dir)
            if filename.endswith(".h"):
                depends.append(relpath)
            elif filename.endswith(".c"):
                sources.append(relpath)

    libraries = [
        ("libbrotli", {
            "include_dirs": [
                "libbrotli/c/include",
                "libbrotli/c/common",
                "libbrotli/c",
                "src/brotlicffi"
            ],
            "depends": depends,
            "sources": sources,
        }),
    ]

cmdclass = {'build_ext': BuildClibBeforeExt}
if sys.version_info > (3,) and platform.python_implementation() == "CPython":
    try:
        import wheel.bdist_wheel
    except ImportError:
        pass
    else:
        # the limited API is only supported on GIL builds as of Python 3.14
        if not bool(sysconfig.get_config_var("Py_GIL_DISABLED")):
            class BDistWheel(wheel.bdist_wheel.bdist_wheel):
                def finalize_options(self):
                    self.py_limited_api = "cp3{}".format(sys.version_info[1])
                    wheel.bdist_wheel.bdist_wheel.finalize_options(self)
            cmdclass['bdist_wheel'] = BDistWheel

setup(
    name="brotlicffi",
    version=__version__,

    description="Python CFFI bindings to the Brotli library",
    long_description=long_description,
    url="https://github.com/python-hyper/brotlicffi",
    license="MIT",

    author="Cory Benfield",
    author_email="cory@lukasa.co.uk",

    setup_requires=[
        "cffi>=1.0.0; python_version<'3.13'",
        "cffi>=1.17.0; python_version>='3.13'",
    ],
    install_requires=[
        "cffi>=1.0.0; python_version<'3.13'",
        "cffi>=1.17.0; python_version>='3.13'",
    ],
    python_requires=">=3.8",
    cffi_modules=["src/brotlicffi/_build.py:ffi"],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    ext_package="brotlicffi",
    libraries=libraries,

    zip_safe=False,

    cmdclass=cmdclass,

    classifiers=[
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        "Programming Language :: Python :: Free Threading :: 2 - Beta",
    ]
)



# -*- coding: utf-8 -*-
"""
test_compatibility
~~~~~~~~~~~~~~~~~~

Tests for names that exist purely for compatibility purposes.
"""
import re
from os.path import abspath, join, dirname
import brotlicffi


def test_compatible_names():
    """
    Encoder modes are also defined as individual top-level names with the same
    names as in brotlimodule.cc from the library.
    """
    assert brotlicffi.MODE_GENERIC is brotlicffi.BrotliEncoderMode.GENERIC
    assert brotlicffi.MODE_TEXT is brotlicffi.BrotliEncoderMode.TEXT
    assert brotlicffi.MODE_FONT is brotlicffi.BrotliEncoderMode.FONT


def test_brotli_version():
    """
    Test that the __version__ starts with the
    Brotli version that's compiled with.
    """
    version_h = join(
        dirname(dirname(abspath(__file__))), "libbrotli/c/common/version.h"
    )
    with open(version_h) as f:
        brotli_versions = dict(
            re.findall(
                r"#define BROTLI_VERSION_(MAJOR|MINOR|PATCH) ([0-9]+)",
                f.read()
            )
        )
        assert brotlicffi.__version__.startswith(
            "%s.%s.%s." % (
                brotli_versions["MAJOR"],
                brotli_versions["MINOR"],
                brotli_versions["PATCH"]
            )
        )
